@@ -10,10 +10,12 @@ const RUN_SCRIPTS = []
 const flatOptions = {
   scriptShell: undefined,
 }
+const defaultLoglevel = 'info'
 const config = {
   json: false,
   parseable: false,
   'if-present': false,
+  loglevel: defaultLoglevel,
 }
 
 const npm = mockNpm({
@@ -26,18 +28,22 @@ const npm = mockNpm({
   output: (...msg) => output.push(msg),
 })
 
+const setLoglevel = (t, level) => {
+  npm.config.set('loglevel', level)
+  t.teardown(() => {
+    npm.config.set('loglevel', defaultLoglevel)
+  })
+}
+
 const output = []
 
-const npmlog = {
-  disableProgress: () => null,
-  level: 'warn',
+const log = {
   error: () => null,
 }
 
 t.afterEach(() => {
   npm.color = false
-  npmlog.level = 'warn'
-  npmlog.error = () => null
+  log.error = () => null
   output.length = 0
   RUN_SCRIPTS.length = 0
   config['if-present'] = false
@@ -55,7 +61,7 @@ const getRS = windows => {
         isServerPackage: require('@npmcli/run-script').isServerPackage,
       }
     ),
-    npmlog,
+    'proc-log': log,
     '../../../lib/utils/is-windows-shell.js': windows,
   })
   return new RunScript(npm)
@@ -355,10 +361,7 @@ t.test('skip pre/post hooks when using ignoreScripts', async t => {
 })
 
 t.test('run silent', async t => {
-  npmlog.level = 'silent'
-  t.teardown(() => {
-    npmlog.level = 'warn'
-  })
+  setLoglevel(t, 'silent')
 
   npm.localPrefix = t.testdir({
     'package.json': JSON.stringify({
@@ -436,12 +439,11 @@ t.test('list scripts', t => {
   })
 
   t.test('silent', async t => {
-    npmlog.level = 'silent'
+    setLoglevel(t, 'silent')
     await runScript.exec([])
     t.strictSame(output, [])
   })
   t.test('warn json', async t => {
-    npmlog.level = 'warn'
     config.json = true
     await runScript.exec([])
     t.strictSame(output, [[JSON.stringify(scripts, 0, 2)]], 'json report')
@@ -719,7 +721,7 @@ t.test('workspaces', t => {
   })
 
   t.test('list no scripts --loglevel=silent', async t => {
-    npmlog.level = 'silent'
+    setLoglevel(t, 'silent')
     await runScript.execWorkspaces([], [])
     t.strictSame(output, [])
   })
@@ -758,7 +760,7 @@ t.test('workspaces', t => {
 
   t.test('missing scripts in all workspaces', async t => {
     const LOG = []
-    npmlog.error = err => {
+    log.error = err => {
       LOG.push(String(err))
     }
     await t.rejects(
@@ -805,7 +807,7 @@ t.test('workspaces', t => {
 
   t.test('missing scripts in some workspaces', async t => {
     const LOG = []
-    npmlog.error = err => {
+    log.error = err => {
       LOG.push(String(err))
     }
     await runScript.execWorkspaces(['test'], ['a', 'b', 'c', 'd'])
@@ -856,7 +858,7 @@ t.test('workspaces', t => {
       '@npmcli/run-script': () => {
         throw new Error('err')
       },
-      npmlog,
+      'proc-log': log,
       '../../../lib/utils/is-windows-shell.js': false,
     })
     const runScript = new RunScript(npm)
@@ -874,7 +876,7 @@ t.test('workspaces', t => {
 
         RUN_SCRIPTS.push(opts)
       },
-      npmlog,
+      'proc-log': log,
       '../../../lib/utils/is-windows-shell.js': false,
     })
     const runScript = new RunScript(npm)
