@@ -1,6 +1,6 @@
 const t = require('tap')
 const spawk = require('spawk')
-const { real: mockNpm } = require('../../fixtures/mock-npm')
+const { load: loadMockNpm } = require('../../fixtures/mock-npm')
 
 spawk.preventUnmatched()
 t.teardown(() => {
@@ -12,22 +12,23 @@ t.teardown(() => {
 // pretty specific internals of runScript
 const makeSpawnArgs = require('@npmcli/run-script/lib/make-spawn-args.js')
 
-t.test('should run stop script from package.json', async t => {
-  const prefix = t.testdir({
-    'package.json': JSON.stringify({
-      name: 'x',
-      version: '1.2.3',
-      scripts: {
-        start: 'node ./test-start.js',
-      },
-    }),
+t.test('should run start script from package.json', async t => {
+  t.plan(2)
+  const { npm } = await loadMockNpm(t, {
+    prefixDir: {
+      'package.json': JSON.stringify({
+        name: 'x',
+        version: '1.2.3',
+        scripts: {
+          start: 'node ./test-start.js',
+        },
+      }),
+    },
+    config: {
+      loglevel: 'silent',
+    },
   })
-  const { Npm } = mockNpm(t)
-  const npm = new Npm()
-  await npm.load()
-  npm.log.level = 'silent'
-  npm.localPrefix = prefix
-  const [scriptShell] = makeSpawnArgs({ path: prefix })
+  const [scriptShell] = makeSpawnArgs({ path: npm.prefix })
   const script = spawk.spawn(scriptShell, (args) => {
     t.ok(args.includes('node ./test-start.js "foo"'), 'ran start script with extra args')
     return true
