@@ -6,7 +6,7 @@ const assert = require('assert')
 const net = require('net')
 const util = require('./core/util')
 const Request = require('./core/request')
-const DispatcherBase = require('./dispatcher-base')
+const Dispatcher = require('./dispatcher')
 const RedirectHandler = require('./handler/redirect')
 const {
   RequestContentLengthMismatchError,
@@ -16,6 +16,8 @@ const {
   RequestAbortedError,
   HeadersTimeoutError,
   HeadersOverflowError,
+  ClientDestroyedError,
+  ClientClosedError,
   SocketError,
   InformationalError,
   BodyTimeoutError,
@@ -43,9 +45,12 @@ const {
   kNoRef,
   kKeepAliveDefaultTimeout,
   kHostHeader,
+  kClosed,
+  kDestroyed,
   kPendingIdx,
   kRunningIdx,
   kError,
+  kOnDestroyed,
   kPipelining,
   kSocket,
   kKeepAliveTimeoutValue,
@@ -58,14 +63,9 @@ const {
   kConnector,
   kMaxRedirections,
   kMaxRequests,
-  kCounter,
-  kClose,
-  kDestroy,
-  kDispatch
+  kCounter
 } = require('./core/symbols')
 
-const kClosedResolve = Symbol('kClosedResolve')
-
 const channels = {}
 
 try {
@@ -81,7 +81,7 @@ try {
   channels.connected = { hasSubscribers: false }
 }
 
-class Client extends DispatcherBase {
+class Client extends Dispatcher {
   constructor (url, {
     maxHeaderSize,
     headersTimeout,
@@ -189,7 +189,10 @@ class Client extends DispatcherBase {
     this[kKeepAliveMaxTimeout] = keepAliveMaxTimeout == null ? 600e3 : keepAliveMaxTimeout
     this[kKeepAliveTimeoutThreshold] = keepAliveTimeoutThreshold == null ? 1e3 : keepAliveTimeoutThreshold
     this[kKeepAliveTimeoutValue] = this[kKeepAliveDefaultTimeout]
+    this[kClosed] = false
+    this[kDestroyed] = false
     this[kServerName] = null
+    this[kOnDestroyed] = []
     this[kResuming] = 0 // 0, idle, 1, scheduled, 2 resuming
     this[kNeedDrain] = 0 // 0, idle, 1, scheduled, 2 resuming
     this[kHostHeader] = `host: ${this[kUrl].hostname}${this[kUrl].port ? `:${this[kUrl].port}` : ''}\r\n`
@@ -198,7 +201,6 @@ class Client extends DispatcherBase {
     this[kStrictContentLength] = strictContentLength == null ? true : strictContentLength
     this[kMaxRedirections] = maxRedirections
     this[kMaxRequests] = maxRequestsPerClient
-    this[kClosedResolve] = null
 
     // kQueue is built up of 3 sections separated by
     // the kRunningIdx and kPendingIdx indices.
@@ -214,15 +216,25 @@ class Client extends DispatcherBase {
     this[kPendingIdx] = 0
   }
 
+  // TODO: Make private?
   get pipelining () {
     return this[kPipelining]
   }
 
+  // TODO: Make private?
   set pipelining (value) {
     this[kPipelining] = value
     resume(this, true)
   }
 
+  get destroyed () {
+    return this[kDestroyed]
+  }
+
+  get closed () {
+    return this[kClosed]
+  }
+
   get [kPending] () {
     return this[kQueue].length - this[kPendingIdx]
   }
@@ -254,68 +266,141 @@ class Client extends DispatcherBase {
     this.once('connect', cb)
   }
 
-  [kDispatch] (opts, handler) {
-    const { maxRedirections = this[kMaxRedirections] } = opts
-    if (maxRedirections) {
-      handler = new RedirectHandler(this, maxRedirections, opts, handler)
+  dispatch (opts, handler) {
+    if (!handler || typeof handler !== 'object') {
+      throw new InvalidArgumentError('handler must be an object')
     }
 
-    const origin = opts.origin || this[kUrl].origin
+    try {
+      if (!opts || typeof opts !== 'object') {
+        throw new InvalidArgumentError('opts must be an object.')
+      }
 
-    const request = new Request(origin, opts, handler)
+      if (this[kDestroyed]) {
+        throw new ClientDestroyedError()
+      }
 
-    this[kQueue].push(request)
-    if (this[kResuming]) {
-      // Do nothing.
-    } else if (util.bodyLength(request.body) == null && util.isIterable(request.body)) {
-      // Wait a tick in case stream/iterator is ended in the same tick.
-      this[kResuming] = 1
-      process.nextTick(resume, this)
-    } else {
-      resume(this, true)
-    }
+      if (this[kClosed]) {
+        throw new ClientClosedError()
+      }
 
-    if (this[kResuming] && this[kNeedDrain] !== 2 && this[kBusy]) {
-      this[kNeedDrain] = 2
+      const { maxRedirections = this[kMaxRedirections] } = opts
+      if (maxRedirections) {
+        handler = new RedirectHandler(this, maxRedirections, opts, handler)
+      }
+
+      const origin = opts.origin || this[kUrl].origin
+
+      const request = new Request(origin, opts, handler)
+
+      this[kQueue].push(request)
+      if (this[kResuming]) {
+        // Do nothing.
+      } else if (util.bodyLength(request.body) == null && util.isIterable(request.body)) {
+        // Wait a tick in case stream/iterator is ended in the same tick.
+        this[kResuming] = 1
+        process.nextTick(resume, this)
+      } else {
+        resume(this, true)
+      }
+
+      if (this[kResuming] && this[kNeedDrain] !== 2 && this[kBusy]) {
+        this[kNeedDrain] = 2
+      }
+    } catch (err) {
+      if (typeof handler.onError !== 'function') {
+        throw new InvalidArgumentError('invalid onError method')
+      }
+
+      handler.onError(err)
     }
 
     return this[kNeedDrain] < 2
   }
 
-  async [kClose] () {
-    return new Promise((resolve) => {
-      if (!this[kSize]) {
-        this.destroy(resolve)
-      } else {
-        this[kClosedResolve] = resolve
-      }
-    })
+  close (callback) {
+    if (callback === undefined) {
+      return new Promise((resolve, reject) => {
+        this.close((err, data) => {
+          return err ? reject(err) : resolve(data)
+        })
+      })
+    }
+
+    if (typeof callback !== 'function') {
+      throw new InvalidArgumentError('invalid callback')
+    }
+
+    if (this[kDestroyed]) {
+      queueMicrotask(() => callback(new ClientDestroyedError(), null))
+      return
+    }
+
+    this[kClosed] = true
+
+    if (!this[kSize]) {
+      this.destroy(callback)
+    } else {
+      this[kOnDestroyed].push(callback)
+    }
   }
 
-  async [kDestroy] (err) {
-    return new Promise((resolve) => {
-      const requests = this[kQueue].splice(this[kPendingIdx])
-      for (let i = 0; i < requests.length; i++) {
-        const request = requests[i]
-        errorRequest(this, request, err)
-      }
+  destroy (err, callback) {
+    if (typeof err === 'function') {
+      callback = err
+      err = null
+    }
 
-      const callback = () => {
-        if (this[kClosedResolve]) {
-          this[kClosedResolve]()
-          this[kClosedResolve] = null
-        }
-        resolve()
-      }
+    if (callback === undefined) {
+      return new Promise((resolve, reject) => {
+        this.destroy(err, (err, data) => {
+          return err ? /* istanbul ignore next: should never error */ reject(err) : resolve(data)
+        })
+      })
+    }
 
-      if (!this[kSocket]) {
-        queueMicrotask(callback)
+    if (typeof callback !== 'function') {
+      throw new InvalidArgumentError('invalid callback')
+    }
+
+    if (this[kDestroyed]) {
+      if (this[kOnDestroyed]) {
+        this[kOnDestroyed].push(callback)
       } else {
-        util.destroy(this[kSocket].on('close', callback), err)
+        queueMicrotask(() => callback(null, null))
       }
+      return
+    }
 
-      resume(this)
-    })
+    if (!err) {
+      err = new ClientDestroyedError()
+    }
+
+    const requests = this[kQueue].splice(this[kPendingIdx])
+    for (let i = 0; i < requests.length; i++) {
+      const request = requests[i]
+      errorRequest(this, request, err)
+    }
+
+    this[kClosed] = true
+    this[kDestroyed] = true
+    this[kOnDestroyed].push(callback)
+
+    const onDestroyed = () => {
+      const callbacks = this[kOnDestroyed]
+      this[kOnDestroyed] = null
+      for (let i = 0; i < callbacks.length; i++) {
+        callbacks[i](null, null)
+      }
+    }
+
+    if (!this[kSocket]) {
+      queueMicrotask(onDestroyed)
+    } else {
+      util.destroy(this[kSocket].on('close', onDestroyed), err)
+    }
+
+    resume(this)
   }
 }
 
@@ -391,6 +476,7 @@ async function lazyllhttp () {
 let llhttpInstance = null
 let llhttpPromise = lazyllhttp()
   .catch(() => {
+    // TODO: Emit warning?
   })
 
 let currentParser = null
@@ -500,6 +586,7 @@ class Parser {
       currentBufferPtr = llhttp.malloc(currentBufferSize)
     }
 
+    // TODO (perf): Can we avoid this copy somehow?
     new Uint8Array(llhttp.memory.buffer, currentBufferPtr, currentBufferSize).set(data)
 
     // Call `execute` on the wasm parser.
@@ -548,10 +635,12 @@ class Parser {
     try {
       try {
         currentParser = this
+        this.llhttp.llhttp_finish(this.ptr) // TODO (fix): Check ret?
       } finally {
         currentParser = null
       }
     } catch (err) {
+      // TODO (fix): What if socket is already destroyed? Error will be swallowed.
       /* istanbul ignore next: difficult to make a test case for */
       util.destroy(this.socket, err)
     }
@@ -693,15 +782,19 @@ class Parser {
       return -1
     }
 
+    // TODO: Check for content-length mismatch from server?
+
     assert(!this.upgrade)
     assert(this.statusCode < 200)
 
+    // TODO: More statusCode validation?
+
     if (statusCode === 100) {
       util.destroy(socket, new SocketError('bad response', util.getSocketInfo(socket)))
       return -1
     }
 
-    /* this can only happen if server is misbehaving */
+    /* istanbul ignore if: this can only happen if server is misbehaving */
     if (upgrade && !request.upgrade) {
       util.destroy(socket, new SocketError('bad upgrade', util.getSocketInfo(socket)))
       return -1
@@ -886,6 +979,7 @@ class Parser {
       util.destroy(socket, new InformationalError('reset'))
       return constants.ERROR.PAUSED
     } else if (!shouldKeepAlive) {
+      // TODO: What if running > 0?
       util.destroy(socket, new InformationalError('reset'))
       return constants.ERROR.PAUSED
     } else if (socket[kReset] && client[kRunning] === 0) {
@@ -985,7 +1079,7 @@ function onSocketClose () {
 
   client[kSocket] = null
 
-  if (client.destroyed) {
+  if (client[kDestroyed]) {
     assert(client[kPending] === 0)
 
     // Fail entire queue.
@@ -1157,14 +1251,14 @@ function resume (client, sync) {
 
 function _resume (client, sync) {
   while (true) {
-    if (client.destroyed) {
+    if (client[kDestroyed]) {
       assert(client[kPending] === 0)
       return
     }
 
-    if (client.closed && !client[kSize]) {
-      client.destroy()
-      return
+    if (client[kClosed] && !client[kSize]) {
+      client.destroy(util.nop)
+      continue
     }
 
     const socket = client[kSocket]
@@ -1385,6 +1479,8 @@ function write (client, request) {
     socket[kBlocking] = true
   }
 
+  // TODO: Expect: 100-continue
+
   let header = `${method} ${path} HTTP/1.1\r\n`
 
   if (typeof host === 'string') {
@@ -1504,6 +1600,7 @@ function writeStream ({ body, client, request, socket, contentLength, header, ex
 
     writer.destroy(err)
 
+    // TODO (fix): Avoid using err.message for logic.
     if (err && (err.code !== 'UND_ERR_INFO' || err.message !== 'reset')) {
       util.destroy(body, err)
     } else {
@@ -1582,6 +1679,7 @@ async function writeIterable ({ body, client, request, socket, contentLength, he
 
   const writer = new AsyncWriter({ socket, request, contentLength, client, expectsPayload, header })
   try {
+    // TODO (fix): What if socket errors while waiting for body?
     // It's up to the user to somehow abort the async iterable.
     for await (const chunk of body) {
       if (socket[kError]) {
@@ -1632,6 +1730,7 @@ class AsyncWriter {
       return true
     }
 
+    // TODO: What if not ended and bytesWritten === contentLength?
     // We should defer writing chunks.
     if (contentLength !== null && bytesWritten + len > contentLength) {
       if (client[kStrictContentLength]) {
@@ -1701,6 +1800,7 @@ class AsyncWriter {
       }
     }
 
+    // TODO (fix): Add comment clarifying what this does?
     if (socket[kParser].timeout && socket[kParser].timeoutType === TIMEOUT_HEADERS) {
       // istanbul ignore else: only for jest
       if (socket[kParser].timeout.refresh) {
