@@ -79,10 +79,11 @@ const DEFAULT_CONDITIONS_SET = new SafeSet(DEFAULT_CONDITIONS);
  * @typedef {string | string[] | Record<string, unknown>} Exports
  * @typedef {'module' | 'commonjs'} PackageType
  * @typedef {{
- *   exports?: ExportConfig;
- *   name?: string;
- *   main?: string;
- *   type?: PackageType;
+ *   pjsonPath: string,
+ *   exports?: ExportConfig,
+ *   name?: string,
+ *   main?: string,
+ *   type?: PackageType,
  * }} PackageConfig
  */
 
@@ -362,6 +363,7 @@ function resolveDirectoryEntry(search) {
 }
 
 const encodedSepRegEx = /%2F|%5C/i;
+let experimentalSpecifierResolutionWarned = false;
 /**
  * @param {URL} resolved
  * @param {string | URL | undefined} base
@@ -376,6 +378,13 @@ function finalizeResolution(resolved, base, preserveSymlinks) {
 
   let path = fileURLToPath(resolved);
   if (getOptionValue('--experimental-specifier-resolution') === 'node') {
+    if (!experimentalSpecifierResolutionWarned) {
+      process.emitWarning(
+        'The Node.js specifier resolution flag is experimental. It could change or be removed at any time.',
+        'ExperimentalWarning');
+      experimentalSpecifierResolutionWarned = true;
+    }
+
     let file = resolveExtensionsWithTryExactName(resolved);
 
     // Directory
