@@ -711,7 +711,7 @@ function callFinal(stream, state) {
       }
     }
   } catch (err) {
-    onFinish(stream, state, err);
+    onFinish(err);
   }
 
   state.sync = false;
